/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package au.gov.qld.health.clientdirectory;

import au.gov.qld.health.clientdirectory.resources.Properties;
import au.gov.qld.health.services.clientdirectory.ClientDirectoryServiceInterface;
import com.stc.eIndex.active.exception.EiException;
import com.stc.eIndex.active.person.DemographicsResultRO;
import com.stc.eIndex.active.person.EiServer;
import com.stc.eIndex.active.person.EnumLocalIdStatus;
import com.stc.eIndex.active.person.EnumPersonStatus;
import com.stc.eIndex.active.person.LocalId;
import com.stc.eIndex.active.person.LocalIdBO;
import com.stc.eIndex.active.person.PersonBO;
import com.stc.eIndex.active.person.PersonSearchResult;
import com.stc.eIndex.active.person.SearchParameters;
import com.stc.eIndex.active.person.Ssn;
import com.stc.eIndex.active.person.Uid;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jws.HandlerChain;
import javax.jws.WebService;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.annotation.security.RolesAllowed;

/**
 *
 * @author BlakeSH
 */
@WebService(serviceName = "clientDirectoryService", portName = "clientDirectoryServicePort", endpointInterface = "au.gov.qld.health.services.clientdirectory.ClientDirectoryServiceInterface", targetNamespace = "http://health.qld.gov.au/services/clientdirectory", wsdlLocation = "META-INF/wsdl/ClientDirectoryService/clientdirectory.wsdl")
@Stateless
@HandlerChain(file = "ClientDirectoryService_handler.xml")
@RolesAllowed("callers")
public class ClientDirectoryService implements ClientDirectoryServiceInterface {

    private static EiServer eiServer;
    private PersonBO personBO;
    private LocalIdBO localIdBO;
    @Resource(mappedName = "clientdirectory/Properties")
    private Properties properties;
    @Resource(mappedName = "clientdirectory/EiServer.properties")
    private Properties eiServerProperties;
    @Resource(mappedName = "clientdirectory/StandardisationCharacters")
    private Properties standardCharacters;

    public au.gov.qld.health.clientdirectory.SearchResponse demographicSearch(au.gov.qld.health.clientdirectory.DemographicSearch parameters) {

        SearchResponse mySearchResponse = new SearchResponse();
        Status myStatus = new Status();
        SearchResult mySearchResult = new SearchResult();

        try {
            XMLGregorianCalendar myXMLDoB;
            DatatypeFactory factory = javax.xml.datatype.DatatypeFactory.newInstance();
            // dateFmt = new SimpleDateFormat("dd/MM/yyyy");
            SimpleDateFormat dateFmt = new SimpleDateFormat("yyyy-MM-dd");
            Demographic myDemographic = new Demographic();
            Demographic.Birth myBirth = new Demographic.Birth();
            Name myName = new Name();
            CardDetails myCardDetails = new CardDetails();
            CardIdentifier myCardIdentifier = null;

            SearchParameters sp = null;
            PersonSearchResult sr = null;
            DemographicsResultRO demog = null;
            int rowcnt = 0;
            int searchpoints = 0;
            int retryCnt;

            sp = new SearchParameters();
            if (parameters.getName().getFirstName() != null && parameters.getName().getFirstName().length() > 0) {
                sp.setFirstName(standardise(parameters.getName().getFirstName()));
                searchpoints += ((parameters.getName().getFirstName().length() > 1) ? Integer.valueOf(properties.get("POINTS_FIRST_NAME")).intValue() : 1);
            }
            if (parameters.getName().getMiddleName() != null && parameters.getName().getMiddleName().length() > 0) {
                sp.setMiddleName(standardise(parameters.getName().getMiddleName()));
                searchpoints+= Integer.valueOf(properties.get("POINTS_MIDDLE_NAME")).intValue();
            }
            if (parameters.getName().getLastName() != null && parameters.getName().getLastName().length() > 0 && parameters.getName().getLastName().substring(0, 1).matches("[a-zA-Z]")) {
                sp.setLastName(standardise(parameters.getName().getLastName()));
                searchpoints += ((parameters.getName().getLastName().length() > 1) ? Integer.valueOf(properties.get("POINTS_LAST_NAME")).intValue() : 1);
            } else {
                myStatus.setCode("VALIDATION_ERROR");
                myStatus.setMessage("Invalid Last Name provided");
                mySearchResponse.setStatus(myStatus);
                return mySearchResponse;
            }
            if (parameters.getDateOfBirth() != null && parameters.getDateOfBirth().length() > 0) {
                try {
                    sp.setDob(new java.sql.Date(dateFmt.parse(parameters.getDateOfBirth()).getTime()));
                } catch (ParseException ex) {
                    myStatus.setCode("VALIDATION_ERROR");
                    myStatus.setMessage("Date of Birth format must be YYYY-MM-DD");
                    mySearchResponse.setStatus(myStatus);
                    return mySearchResponse;
                }
                searchpoints += Integer.valueOf(properties.get("POINTS_DOB")).intValue();
            }
            if (parameters.getSex() != null && parameters.getSex().length() > 0) {
                sp.setGender(parameters.getSex());
                searchpoints+= Integer.valueOf(properties.get("POINTS_SEX")).intValue();
            }
            if (parameters.getMedicare() != null && parameters.getMedicare().length() > 0) {
                if (!validateMedicare(parameters.getMedicare())) {
                    myStatus.setCode("VALIDATION_ERROR");
                    myStatus.setMessage("Invalid Medicare number provided");
                    mySearchResponse.setStatus(myStatus);
                    return mySearchResponse;
                }
                sp.setSsn(new Ssn(parameters.getMedicare().substring(0, 9)));
                searchpoints += Integer.valueOf(properties.get("POINTS_MEDICARE")).intValue();
            }

            if (searchpoints < Integer.valueOf(properties.get("POINTS_THRESHOLD")).intValue()) {
                myStatus.setCode("SEARCH_POINTS_ERROR");
                myStatus.setMessage("Minimum search criteria not met");
                mySearchResponse.setStatus(myStatus);
                return mySearchResponse;
            }

            for (retryCnt = 0; retryCnt < Integer.valueOf(properties.get("MAX_RETRY")).intValue() + 1; retryCnt++) {
                try {
                    sr = personBO.searchAlpha(sp);
                    break; // Leave the retry loop
                } catch (java.sql.SQLException se) {
                    if (retryCnt == (Integer.valueOf(properties.get("MAX_RETRY")).intValue() - 1)) {
                        //log("Max retries reached on SQL/Connection error.  Forcing close of connection pool and trying again.");
                        try {
                            eiServer.closeAllConnections();
                        } catch (Exception ex) {
                        } // Close all connections and try again
                    } else if (retryCnt == Integer.valueOf(properties.get("MAX_RETRY")).intValue()) {
                        throw se;
                    }
                }
            }


            if (sr != null && sr.size() > 0) {
                while (sr.next() && rowcnt < Integer.valueOf(properties.get("MAX_CANDIDATES")).intValue()) {
                    mySearchResult = new SearchResult();
                    myDemographic = new Demographic();
                    myBirth = new Demographic.Birth();
                    myName = new Name();
                    myCardDetails = new CardDetails();
                    myCardIdentifier = new CardIdentifier();
                    demog = sr.getResultRow();
                    System.out.println("Current Status: " + demog.getStatus().toString());
                    if (demog.getStatus().equals(EnumPersonStatus.ACTIVE)) {
                    mySearchResult.setCdn(demog.getUid().toString());
                    myName.setLastName(demog.getLastName());
                    myName.setFirstName(demog.getFirstName());
                    if (demog.getMiddleName() != null) {
                        myName.setMiddleName(demog.getMiddleName());
                    }
                    myDemographic.setName(myName);
                    if (demog.getDob() != null) {
                        myXMLDoB = factory.newXMLGregorianCalendar(demog.getDob().toString().replace(" ", "T"));
                        myBirth.setDateOfBirth(myXMLDoB);
                    }
                    myDemographic.setBirth(myBirth);
                    myDemographic.setSex(demog.getGender());
                    if (demog.getSsn() != null && !demog.getSsn().equals("")) {
                        myCardIdentifier.setCardNo(demog.getSsn().toString());
                        myCardDetails.setMedicare(myCardIdentifier);
                        myDemographic.setCardDetails(myCardDetails);
                    }
                        mySearchResult.setWeight(demog.getComparisonScore());
                    mySearchResult.setDemographic(myDemographic);
                    mySearchResponse.getResult().add(mySearchResult);
                    rowcnt++;
                }
            }
            }

            myStatus.setCode("OK");
            myStatus.setMessage("Demographic Search Complete");
            mySearchResponse.setStatus(myStatus);
            return mySearchResponse;
        } catch (Exception ex) {
            String s = null;
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw, true);
            ex.printStackTrace(pw);
            s = sw.toString();
            myStatus.setCode("SERVER_ERROR");
            myStatus.setMessage("Problem reading search results: " + s);
            mySearchResponse.setStatus(myStatus);
            return mySearchResponse;
        }
    }

    @SuppressWarnings(value = "unchecked")
  public au.gov.qld.health.clientdirectory.IdentifierResponse getIdentifierList(au.gov.qld.health.clientdirectory.IdentifierSearch parameters) {

        IdentifierResponse myIdentifierResponse = new IdentifierResponse();
        Status myStatus = new Status();
        ClientDirectoryIdentifier myCDID = new ClientDirectoryIdentifier();

        try {
            Identifier myIdentifier = new Identifier();
            String checkStatus = "";

            if (parameters.getIdentifier() != null) {

                Uid uid = null;
                com.stc.eIndex.active.person.Person person = new com.stc.eIndex.active.person.Person();

                if (parameters.getIdentifier().getClientDirectoryIdentifier() != null && parameters.getIdentifier().getClientDirectoryIdentifier().getCdn() != null && !parameters.getIdentifier().getClientDirectoryIdentifier().getCdn().equals("")) {
                    uid = this.getUID(parameters.getIdentifier().getClientDirectoryIdentifier().getCdn());
                } else if (parameters.getIdentifier().getLocalIdentifier() != null && parameters.getIdentifier().getLocalIdentifier().getLocalId() != null && !parameters.getIdentifier().getLocalIdentifier().getLocalId().equals("")) {
                    uid = this.getUID(parameters.getIdentifier().getLocalIdentifier().getFacility(), parameters.getIdentifier().getLocalIdentifier().getLocalId());
                }

                if (uid == null) {
                    myStatus.setCode("DATA_NOT_FOUND");
                    myStatus.setMessage("Invalid UID or Facility/Local ID provided");
                    myIdentifierResponse.setStatus(myStatus);
                    return myIdentifierResponse;
                }

                // Local ID Attributes field
                myCDID.setCdn(uid.toString());
                myIdentifier.setClientDirectoryIdentifier(myCDID);
                checkStatus = (parameters.getIdentifier().getClientDirectoryIdentifier() != null) ? parameters.getIdentifier().getClientDirectoryIdentifier().getStatus() : parameters.getIdentifier().getLocalIdentifier().getStatus();
                this.addIdentifierList(myIdentifier.getLocalIdentifier(), localIdBO.getLocalIdEnumeration(uid), checkStatus, parameters.getFacilityFilter());

                myIdentifierResponse.setIdentifier(myIdentifier);

                myStatus.setCode("OK");
                myStatus.setMessage("Get Identifier List Complete");
                myIdentifierResponse.setStatus(myStatus);
            } else {
                myStatus.setCode("DATA_NOT_FOUND");
                myStatus.setMessage("Invalid UID or Facility/Local ID provided");
                myIdentifierResponse.setStatus(myStatus);
            }
            return myIdentifierResponse;
        } catch (Exception ex) {
            Logger.getLogger("au.gov.qld.health.services.clientdirectory.log").log(Level.SEVERE, null, ex);
            String s = null;
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw, true);
            ex.printStackTrace(pw);
            s = sw.toString();
            myStatus.setCode("SERVER_ERROR");
            myStatus.setMessage("Problem reading search results: " + s);
            myIdentifierResponse.setStatus(myStatus);
            return myIdentifierResponse;
        }
    }

    @SuppressWarnings(value = "unchecked")
  public au.gov.qld.health.clientdirectory.PersonResponse getPerson(au.gov.qld.health.clientdirectory.IdentifierSearch parameters) {

        PersonResponse myPersonResponse = new PersonResponse();
        Status myStatus = new Status();
        ClientDirectoryIdentifier myCDID = new ClientDirectoryIdentifier();

        try {
            XMLGregorianCalendar myXMLDoB;
            XMLGregorianCalendar myXMLDoD;
            DatatypeFactory factory = javax.xml.datatype.DatatypeFactory.newInstance();
            Person myPerson = new Person();
            Demographic myDemographic = new Demographic();
            Demographic.Birth myDoB = new Demographic.Birth();
            Demographic.Death myDeath = new Demographic.Death();
            Address myAddress = new Address();
            Telephone myPhone = new Telephone();
            AlternateIdentifier myAltId = new AlternateIdentifier();
            Name myName = new Name();
            CardDetails myCardDetails = new CardDetails();
            CardIdentifier myCardId = null;
            Identifier myIdentifier = new Identifier();
            String checkStatus = "";

            if (parameters.getIdentifier() != null) {
                Enumeration enumeration = null;
                com.stc.eIndex.active.person.Person person = new com.stc.eIndex.active.person.Person();
                com.stc.eIndex.active.person.Demographics demog = null;
                com.stc.eIndex.active.person.Address addr = null;
                com.stc.eIndex.active.person.Phone phone = null;
                com.stc.eIndex.active.person.Alias alias = null;
                com.stc.eIndex.active.person.AuxId auxid = null;
                com.stc.eIndex.active.person.Uid uid = null;

                uid = this.findPerson(parameters.getIdentifier(), person, myStatus);
                if (uid == null) {
                    myPersonResponse.setStatus(myStatus);
                    return myPersonResponse;
                }

                // Demographic Attributes field
                demog = person.getDemographics();

                // Local ID Attributes field
                myCDID.setCdn(uid.toString());
                myCDID.setStatus(person.getStatus().toString());
                myIdentifier.setClientDirectoryIdentifier(myCDID);
                checkStatus = (parameters.getIdentifier().getClientDirectoryIdentifier() != null) ? parameters.getIdentifier().getClientDirectoryIdentifier().getStatus() : parameters.getIdentifier().getLocalIdentifier().getStatus();
                this.addIdentifierList(myIdentifier.getLocalIdentifier(), person.getLocalIdEnumeration(), checkStatus, parameters.getFacilityFilter());

                myPerson.setIdentifier(myIdentifier);

                myName.setLastName(demog.getLastName());
                myName.setFirstName(demog.getFirstName());
                myName.setMiddleName(demog.getMiddleName());

                myDemographic.setName(myName);

                myXMLDoB = factory.newXMLGregorianCalendar(demog.getDob().toString().replace(" ", "T"));
                myDoB.setEstimated(demog.getSuffix());
                myDoB.setDateOfBirth(myXMLDoB);
                myDemographic.setBirth(myDoB);

                myDemographic.setSex(demog.getGender());
                myDemographic.setCountryOfBirth(demog.getPobCountry());
                myDemographic.setMaritalStatus(demog.getMaritalStatus());
                myDemographic.setReligion(demog.getReligion());
                myDemographic.setLanguage(demog.getLanguage());
                myDemographic.setTitle(demog.getTitle());
                myDemographic.setIndigenousStatus(demog.getRace());
                myDemographic.setAssi(demog.getEthnic());

                myDeath.setDeceased(demog.getDeath());
                if (demog.getDateOfDeath() != null) {
                    myXMLDoD = factory.newXMLGregorianCalendar(demog.getDateOfDeath().toString().replace(" ", "T"));
                    myDeath.setDateOfDeath(myXMLDoD);
                }
                myDemographic.setDeath(myDeath);

                myCardId = new CardIdentifier();
                if (demog.getSsn() != null && !demog.getSsn().equals("")) {
                    myCardId.setCardNo(demog.getSsn().toString());
                    myCardId.setSuffix(demog.getNationality());
                    myCardId.setExpiryDate(demog.getString1());
                    myCardDetails.setMedicare(myCardId);
                }

                if (demog.getRepatriationNumber() != null && !demog.getRepatriationNumber().equals("")) {
                    myCardId = new CardIdentifier();
                    myCardId.setCardNo(demog.getRepatriationNumber());
                    myCardId.setSuffix(demog.getVeteranStatus());
                    myCardId.setExpiryDate(demog.getString3());
                    myCardDetails.setDva(myCardId);
                }

                if (demog.getPensionNumber() != null && !demog.getPensionNumber().equals("")) {
                    myCardId = new CardIdentifier();
                    myCardId.setCardNo(demog.getPensionNumber());
                    myCardId.setSuffix(demog.getDistrictOfResidence());
                    myCardId.setExpiryDate(demog.getString2());
                    myCardDetails.setPension(myCardId);
                }

                if (demog.getMotherName() != null && !demog.getMotherName().equals("")) {
                    myCardId = new CardIdentifier();
                    myCardId.setCardNo(demog.getMotherName());
                    myCardId.setExpiryDate(demog.getString4());
                    myCardDetails.setSafetyNet(myCardId);
                }

                if (demog.getFatherName() != null && !demog.getFatherName().equals("")) {
                    myCardId = new CardIdentifier();
                    myCardId.setCardNo(demog.getFatherName());
                    myCardId.setExpiryDate(demog.getString5());
                    myCardDetails.setHealthCare(myCardId);
                }

                myDemographic.setCardDetails(myCardDetails);

                //              Available data not returned
//              demog.getMilitaryBranch()
//              demog.getMotherMaidenName()
//              demog.getPersonCategoryCode()
//              demog.getMilitaryRank()
//              demog.getVipFlag()
//              demog.getStatus()
                myPerson.setDemographic(myDemographic);
                myPersonResponse.setPerson(myPerson);

                if ((enumeration = person.getAddressEnumeration()).hasMoreElements()) {
                    while (enumeration.hasMoreElements()) {
                        addr = (com.stc.eIndex.active.person.Address) enumeration.nextElement();
                        myAddress = new Address();
                        myAddress.setAddress1(addr.getAddress1());
                        myAddress.setAddress2(addr.getAddress2());
                        myAddress.setType(addr.getAddressType());
                        myAddress.setCity(addr.getCity());
                        myAddress.setCountry(addr.getCountry());
                        myAddress.setPostCode(addr.getPostalCode());
                        myAddress.setState(addr.getStateOrProvince());
                        myPersonResponse.getPerson().getAddress().add(myAddress);
                    }
                }

                if ((enumeration = person.getPhoneEnumeration()).hasMoreElements()) {
                    while (enumeration.hasMoreElements()) {
                        phone = (com.stc.eIndex.active.person.Phone) enumeration.nextElement();
                        myPhone = new Telephone();
                        myPhone.setPhoneNo(phone.getPhoneNumber());
                        myPhone.setType(phone.getPhoneType());
                        myPersonResponse.getPerson().getTelephone().add(myPhone);
                    }
                }

                if ((enumeration = person.getAuxIdEnumeration()).hasMoreElements()) {
                    while (enumeration.hasMoreElements()) {
                        auxid = (com.stc.eIndex.active.person.AuxId) enumeration.nextElement();
                        myAltId = new AlternateIdentifier();
                        myAltId.setIdentifier(auxid.getId());
                        myAltId.setName(auxid.getAuxIdType());
                        myPersonResponse.getPerson().getAlternateId().add(myAltId);
                    }
                }

                if ((enumeration = person.getAliasEnumeration()).hasMoreElements()) {
                    while (enumeration.hasMoreElements()) {
                        alias = (com.stc.eIndex.active.person.Alias) enumeration.nextElement();
                        myName = new Name();
                        myName.setFirstName(alias.getFirstName());
                        myName.setLastName(alias.getLastName());
                        myName.setMiddleName(alias.getMiddleName());
                        myPersonResponse.getPerson().getAlias().add(myName);
                    }
                }

                myStatus.setCode("OK");
                myStatus.setMessage("Get Person Complete");
                myPersonResponse.setStatus(myStatus);
            } else {
                myStatus.setCode("DATA_NOT_FOUND");
                myStatus.setMessage("Invalid UID or Facility/Local ID provided");
                myPersonResponse.setStatus(myStatus);
            }
        } catch (DatatypeConfigurationException ex) {
            Logger.getLogger("au.gov.qld.health.services.clientdirectory.log").log(Level.SEVERE, null, ex);
            myStatus.setCode("SERVER_ERROR");
            myStatus.setMessage("DatatypeConfigurationException Exception: " + ex.getMessage());
        } catch (SQLException ex) {
            Logger.getLogger("au.gov.qld.health.services.clientdirectory.log").log(Level.SEVERE, null, ex);
            myStatus.setCode("SERVER_ERROR");
            myStatus.setMessage("SQLException Exception: " + ex.getMessage());
        } catch (EiException ex) {
            Logger.getLogger("au.gov.qld.health.services.clientdirectory.log").log(Level.SEVERE, null, ex);
            myStatus.setCode("SERVER_ERROR");
            myStatus.setMessage("EiException Exception: " + ex.getMessage());
        } catch (Exception ex) {
            Logger.getLogger("au.gov.qld.health.services.clientdirectory.log").log(Level.SEVERE, null, ex);
            myStatus.setCode("SERVER_ERROR");
            myStatus.setMessage("Exception: " + ex.getMessage());
        }
        return myPersonResponse;
    }

    private Uid getUID(String fac, String urn) throws Exception {

        Uid uid = null;
        int retryCnt;

        for (retryCnt = 0; retryCnt < Integer.valueOf(properties.get("MAX_RETRY")).intValue() + 1; retryCnt++) {
            try {
                uid = personBO.lookupUid(fac, urn, EnumLocalIdStatus.getEnumeration("A"));
                if (uid == null) {
                    uid = personBO.lookupUid(fac, urn, EnumLocalIdStatus.getEnumeration("M"));
                }
                break; // Leave the retry loop
            } catch (java.sql.SQLException se) {
                if (retryCnt == (Integer.valueOf(properties.get("MAX_RETRY")).intValue() - 1)) {
                    //log("Max retries reached on SQL/Connection error.  Forcing close of connection pool and trying again.");
                    try {
                        eiServer.closeAllConnections();
                    } catch (Exception ex) {
                    } // Close all connections and try again
                } else if (retryCnt == Integer.valueOf(properties.get("MAX_RETRY")).intValue()) {
                    throw se;
                }
            }
        }

        return uid;
    }

    private Uid getUID(String uidStr) throws Exception {
        Uid uid = null;

        try {
            uid = new Uid(uidStr);
            Enumeration e = localIdBO.getLocalIdEnumeration(uid);
            if (e == null || !e.hasMoreElements()) {
                return null;
            }
            LocalId loc = (LocalId) e.nextElement();
            uid = this.getUID(loc.getSystemCode(), loc.getLocalId());
        } catch (Exception ex) {
            if (ex instanceof NullPointerException) {
                return null;
            } else {
                throw ex;
            }
        }
        return uid;
    }

    private void addIdentifierList(List<LocalIdentifier> localIdentifiers, Enumeration<com.stc.eIndex.active.person.LocalId> localIds, String checkStatus, List<String> facilityCodes) throws Exception {

        LocalId lid = null;
        LocalIdentifier myIDList = null;

        if (checkStatus == null || checkStatus.equals("")) {
            checkStatus = "*";
        }
        while (facilityCodes.contains("")) {
            facilityCodes.remove("");
        }
        if (localIds.hasMoreElements()) {
            while (localIds.hasMoreElements()) {
                lid = localIds.nextElement();
                if (facilityCodes == null || facilityCodes.isEmpty() || facilityCodes.contains(lid.getSystemCode())) {
                    if (checkStatus.equals("*") || checkStatus.equalsIgnoreCase(lid.getStatus().toString())) {
                        myIDList = new LocalIdentifier();
                        myIDList.setLocalId(lid.getLocalId());
                        myIDList.setFacility(lid.getSystemCode());
                        myIDList.setStatus(lid.getStatus().toString());
                        localIdentifiers.add(myIDList);
                    }
                }
            }
        }
    }

    private com.stc.eIndex.active.person.Uid findPerson(IdentifierSearch.Identifier id, com.stc.eIndex.active.person.Person person, Status status) {

        com.stc.eIndex.active.person.Uid uid = null;

        try {
            if (id.getClientDirectoryIdentifier() != null && id.getClientDirectoryIdentifier().getCdn() != null && !id.getClientDirectoryIdentifier().getCdn().equals("")) {
                uid = getUID(id.getClientDirectoryIdentifier().getCdn());
            } else if (id.getLocalIdentifier() != null && id.getLocalIdentifier().getFacility() != null && id.getLocalIdentifier().getLocalId() != null && !id.getLocalIdentifier().getFacility().equals("") && !id.getLocalIdentifier().getLocalId().equals("")) {
                uid = getUID(id.getLocalIdentifier().getFacility(), id.getLocalIdentifier().getLocalId());
            } else {
                status.setCode("DATA_NOT_FOUND");
                status.setMessage("No UID or Facility/Local ID provided");
                return null;
            }

            if (uid == null) {
                status.setCode("DATA_NOT_FOUND");
                status.setMessage("Invalid UID or Facility/Local ID provided");
                return null;
            }

            try {
                personBO.loadPerson(uid, person);
            } catch (EiException.InvalidParameter ex) {
                // If the uid isn't valid, we'll get this exception
                status.setCode("SERVER_ERROR");
                status.setMessage("EiException.InvalidParameter: " + ex.getMessage());
                return null;
            }
        } catch (Exception ex) {
            Logger.getLogger("au.gov.qld.health.services.clientdirectory.log").log(Level.SEVERE, null, ex);
            status.setCode("DATA_NOT_FOUND");
            status.setMessage("Exception: " + ex.getMessage());
            return null;
        }
        return uid;
    }

    private boolean validateMedicare(String medicare) {
        if (medicare.length() < 9) {
            return false;
        }
        medicare = medicare.substring(0, 9);

        if (Integer.valueOf(medicare.substring(0, 1)).intValue() < 2 || Integer.valueOf(medicare.substring(0, 1)).intValue() > 6) {
            return false;
        }

        int total = 0;
        for (int i = 0; i < 8; i++) {
            switch (i) {
                case 0:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 1;
                    break;
                case 1:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 3;
                    break;
                case 2:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 7;
                    break;
                case 3:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 9;
                    break;
                case 4:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 1;
                    break;
                case 5:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 3;
                    break;
                case 6:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 7;
                    break;
                case 7:
                    total += Integer.valueOf(medicare.substring(i, i + 1)).intValue() * 9;
                    break;
                default:
                    break;
            }
        }
        if (!(String.valueOf(total).endsWith(medicare.substring(8, 9)))) {
            return false;
        }
        return true;
    }

    private String standardise(String input) {
        
        for(Map.Entry<String, String> entry : standardCharacters.getAll().entrySet()) {
            input = input.replace((char)(Integer.valueOf(entry.getKey()) + '0'), entry.getValue().charAt(0));
        }
        return input;
    }

    @PostConstruct
    public void postConstruct() {
        try {
            java.util.Properties props = new java.util.Properties();
            props.putAll(eiServerProperties.getAll());

            synchronized (this) {
                if (eiServer == null) {
                    eiServer = new EiServer(props);
                }
            }
            personBO = eiServer.getEiBOFactory().getPersonBOInstance();
            localIdBO = eiServer.getEiBOFactory().getLocalIdBOInstance();
        } catch (SQLException ex) {
        } catch (EiException exe) {
        }
    }

    @PreDestroy
    public void preDestroy() {
        personBO = null;
        localIdBO = null;
    }
}