/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package au.gov.qld.health.clientdirectory.handlers;

import au.gov.qld.health.clientdirectory.auditing.Audit;
import au.gov.qld.health.clientdirectory.auditing.AuditEvent;
import au.gov.qld.health.clientdirectory.auditing.AuditException;
import java.io.IOException;
import java.security.Principal;
import java.util.GregorianCalendar;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.EJB;
import javax.security.auth.Subject;
import javax.servlet.http.HttpServletRequest;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

/**
 * @TODO Move this to the AuditLoggingFramework
 * @TODO Complete Fault logging and auditing
 * @TODO Continue to test Subject access
 * @TODO fix filename for log
 * 
 * @author Shaneb
 */
public class ServiceHandler implements SOAPHandler {

    private static Logger logger;
    
    static {
        try {
            // Create an appending file handler
            boolean append = true;
            FileHandler exceptionHandler = new FileHandler("exception.log", append);

            // Add to the desired logger
            logger = Logger.getLogger("au.gov.qld.health.services.clientdirectory.log");
            logger.addHandler(exceptionHandler);
        } catch (IOException e) {
            Logger.getLogger("").severe("Failed to initialise exception log");
        }
      
    }
    
    // Inject Audit implementation into Handler instance by the container
    @EJB(name="auditImpl", beanInterface=au.gov.qld.health.clientdirectory.auditing.Audit.class)
    private Audit a;
    private AuditEvent event;
    
    public Set getHeaders() {
        return new TreeSet();
    }

    public boolean handleMessage(MessageContext context) {
        Boolean outboundProperty = (Boolean)context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        if(outboundProperty.booleanValue()){
            // Set this message appropriately
            event.setResult("SUCCESS");
            try {
                a.audit(event);
            } catch(AuditException ex) {
                logger.log(Level.SEVERE, ex.getMessage());
            }
        } else {
            // Extract caller details from MessageContext
            SOAPMessageContext soapCtx = (SOAPMessageContext)context;
            SOAPMessage msg = soapCtx.getMessage();
            
            try {
                if(msg.getSOAPHeader() != null) {
                    // Create a new AuditEvent instance
                    event = new AuditEvent();

                    // Get the identifier of the caller from the Subject of the requester
                    Subject subj = com.sun.xml.wss.SubjectAccessor.getRequesterSubject(context);
                    if(subj != null) {
                        Set<Principal> principals = subj.getPrincipals();
                        if(principals.iterator().hasNext()) {
                            Principal principal = principals.iterator().next();
                            if(principal != null) {
                                String identifier = principal.getName();
                                if(identifier != null) {
                                    event.setIdentifier(identifier);
                                }
                            }
                        }
                    }
                    
                    // Get remote IP Address from Servlet Request of Web Service Call.
                    HttpServletRequest request = (HttpServletRequest)context.get(MessageContext.SERVLET_REQUEST);
                    event.setIpAddress(request.getRemoteAddr());

                    // Set the parameters to the content of the SOAP Body
                    event.setParameters(msg.getSOAPBody().getFirstChild());
System.out.println("here");                    
                    // Set the Request Time to the current date and time
                    event.setRequestTime(GregorianCalendar.getInstance().getTime());
                }
            } catch(SOAPException ex) {
                logger.log(Level.SEVERE, "SOAP Processing Exception: " + ex.getMessage());
            } catch(NullPointerException npe) {
                logger.log(Level.SEVERE, "Null Pointer Exception: " + npe.getMessage());
            } catch(Exception e) {
                logger.log(Level.SEVERE, "General Exception: " + e.getMessage());
            }
        }
        return true;
    }

    public boolean handleFault(MessageContext context) {
        Boolean outboundProperty = (Boolean)context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        if(outboundProperty.booleanValue()){
            // Log the exception
            
            // Set this message appropriately, checking for auditable fault, eg. Security Fault
            if(true) {
                event.setResult("FAULT");
                try {
                    a.audit(event);
                } catch(AuditException ex) {
                    logger.log(Level.SEVERE, "Audit Exception: " + ex.getMessage());
                }    
            }
        }
        return true;
    }

    public void close(MessageContext context) {
        return;
    }
    
}
